from typing import Mapping, Any, Tuple, Optional, Union
import json

headers = {'Content-Type': 'application/json; charset=utf-8'}

def buildBadRequestResponse(message: Optional[str]) -> Tuple[str, int,  Mapping[str, str]]:
    return (
        json.dumps({"status": False, "reason": "Bad Request", "message": message}),
        400,
        headers
    )


def buildValidResponse(jsonDict: Union[str, Mapping[str, Any]]) -> Tuple[str, int, Mapping[str, str]]:
    return json.dumps({"status": True, "data": jsonDict}, indent=4), 200, headers


def buildNotFoundResponse(message: Optional[str]) -> Tuple[str, int, Mapping[str, str]]:
    return json.dumps({"status": False, "reason": "Not Found", "message": message}), 200, headers

def buildErrorResponse(jsonDict: Union[str, Mapping[str, Any]]) -> Tuple[str, int, Mapping[str, str]]:
    return json.dumps({"status": False, "reason": "Error", "message": jsonDict}), 500, headers


from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Session, scoped_session
from sqlalchemy.exc import OperationalError
from time import sleep
import os
import logging

Base = declarative_base()

__session_factory = None
__engine = None

def connect(cnt=0) -> None:
    global __session_factory
    global __engine

    try:
        if not __engine:
            __engine = create_engine(
                "postgresql://postgres:%s@db:5432/" % os.environ["POSTGRES_PASSWORD"],
                max_overflow=1000  # High number but not setting to -1
            )
        if not __session_factory:
            __session_factory = sessionmaker(bind=__engine)
            Base.metadata.create_all(__engine)
    except OperationalError as e:
        if cnt > 30:
            raise e
        sleep(1)
        logging.info(
            "Failed to connect to DB! Assuming this is a timeout error and waiting 1 second before retrying!"
        )
        return connect(cnt=cnt + 1)

def getSession() -> Session:
    connect()
    return scoped_session(__session_factory)

def getEngine():
    connect()
    return __engine


import logging
from time import time

def short_str(any) -> str:
    s = str(any)
    if len(s) < 20:
        return s
    else:
        return s[:10] + " ... " + s[-10:]

def benchmark(f):
    def benchmarked_f(*args, **kwargs):
        startTime = time()
        res = f(*args, **kwargs)
        timeToRun = time() - startTime
        args = [short_str(x) for x in args]
        argStr = ", ".join(args) + ", " + ", ".join([key + "=" + str(val) for key, val in kwargs.items()])
        logging.info("Took %s seconds to run %s.%s(%s)" % (str(timeToRun), f.__module__, f.__name__, argStr))
        return res
    return benchmarked_f



from distutils.core import setup

setup(
    name='tulgey',
    version='0.1.12',
    author='David Dworken',
    author_email='david@daviddworken.com',
    packages=['tulgey', 'tulgey.db', 'tulgey.utils'],
    url='http://pypi.python.org/pypi/tulgey/',
    license='LICENSE.txt',
    description='tulgey python utils',
)



import redis
import datetime

from typing import Optional

STATS_PREFIX = "STATS"
SEP = "_"

def incrStat(statName: str, redisConnection: redis.StrictRedis = None) -> None:
    if not redisConnection:
        redisConnection = redis.StrictRedis('redis')
    date = datetime.datetime.today().strftime("%Y-%m-%d")
    redisConnection.incr(STATS_PREFIX + SEP + statName + SEP + date)

def getStatToday(statName: str, redisConnection: redis.StrictRedis = None) -> Optional[int]:
    if not redisConnection:
        redisConnection = redis.StrictRedis('redis')
    date = datetime.datetime.today().strftime("%Y-%m-%d")
    ret = redisConnection.get(STATS_PREFIX + SEP + statName + SEP + date)
    if ret:
        return int(ret)
    else:
        return None


from typing import Mapping, Any
from dataclasses import dataclass
from tulgey.PortScanResult import PortScanResult
from tulgey.utils import tulgeyDatetime
from datetime import datetime

@dataclass
class IPScanResult:
    ip: str
    isUp: bool
    ports: Mapping[int, PortScanResult]
    scannedPorts: str
    dt: datetime

    def toJsonDict(self) -> Mapping[str, Any]:
        return {
            "ip": self.ip,
            "isUp": self.isUp,
            "ports": [psr.toJsonDict() for psr in self.ports.values()],
            "scannedPorts": self.scannedPorts,
            "datetime": tulgeyDatetime.toString(self.dt),
        }

    @staticmethod
    def fromJsonDict(jsonDict) -> "IPScanResult":
        return IPScanResult(
            jsonDict["ip"],
            jsonDict["isUp"],
            {PortScanResult.fromJsonDict(psrjd).port:PortScanResult.fromJsonDict(psrjd) for psrjd in jsonDict["ports"]},
            jsonDict["scannedPorts"],
            tulgeyDatetime.fromString(jsonDict["datetime"]),
        )
